varying mediump vec2 vTexCoordOut; 

#ifdef VS
attribute  vec2 Position;
attribute  vec2 TexCoord0; 


void main(void) {
	gl_Position = vec4(Position.xy,0,1.0);
	vTexCoordOut=TexCoord0;
}
#endif
#ifdef FS

uniform mediump vec4 resolution; 
uniform lowp sampler2D Tex;

mediump float Gaussian (mediump float x, mediump float deviation)
{
	return (1.0 / sqrt(2.0 * 3.141592 * deviation)) * exp(-((x * x) / (2.0 * deviation)));	
}

const mediump float BlurStrength = 1.0;
const mediump float BlurScale = 1.25;
const int BlurAmount = 7;

void main(void) 
{
	mediump float halfBlur = float(BlurAmount) * 0.5;
	mediump vec4 colour = vec4(0.0);
	mediump vec4 texColour = vec4(0.0);
	
	// Gaussian deviation
	mediump float deviation = halfBlur * 0.35;
	deviation *= deviation;
	mediump float strength = 1.0 - BlurStrength;
	
	mediump vec2 TexelSize = vec2(1.0 / resolution.x, 1.0 / resolution.y);

#ifndef _Vertical
	// Horizontal blur
	for (int i = 0; i < BlurAmount; ++i)
	{
		mediump float offset = float(i) - halfBlur;
		texColour = texture2D(Tex, vTexCoordOut + vec2(offset * TexelSize.x * BlurScale, 0.0)) * Gaussian(offset * strength, deviation);
		colour += texColour;
	}
#else
	// Vertical blur
	for (int i = 0; i < BlurAmount; ++i)
	{
		mediump float offset = float(i) - halfBlur;
		texColour = texture2D(Tex, vTexCoordOut + vec2(0.0, offset * TexelSize.y * BlurScale)) * Gaussian(offset * strength, deviation);
		colour += texColour;
	}
#endif
	
	// Apply colour
	gl_FragColor = clamp(colour, 0.0, 1.0);
	gl_FragColor.w = 1.0;
	
	//gl_FragColor = texture2D(Tex, vTexCoordOut);
}

#endif
