#pragma include "common_vs.glh"
#pragma include "common_fs.glh"

varying lowp vec3 vNormal; 
varying mediump vec2 vTexCoord0; 

#ifdef _TEX1
varying mediump vec2 vTexCoord1; 
#endif
varying mediump vec4 vPosition;
#ifdef _COLOR 
varying lowp vec4 vColor;
#endif

#ifdef _DISTANCE_FOG
varying lowp float vDistFog;
#endif


#ifdef VS


uniform  highp mat4 MatWorld; // 4 -> 124
uniform  highp mat4 MatViewProjection; // 4 -> 120
uniform  mediump vec4 DuDvSpeed; 
uniform  mediump vec4 Time; //TODO floata dac

void main(void) 
{
	highp vec4 pos = aPosition;
	pos.w=1.0;
	
	vTexCoord0 = aTexCoord0;
	
	#ifdef _COLOR
		vColor=aColor;
	#endif
	#ifdef _TEX1
		vTexCoord1 =aTexCoord1;
	#endif
	
	
	#ifdef _DUDV 
		vTexCoord0+=mod(DuDvSpeed.xy*Time.xx,2.0);
	#endif 
	
	mediump mat3 world3 = mat3( MatWorld[0].xyz,MatWorld[1].xyz,MatWorld[2].xyz );
	
	#ifdef _ANIM
		highp vec4  nPos =  
					(BoneTransform(aBoneId.x,pos)*aBoneWeight.x+
					 BoneTransform(aBoneId.y,pos)*aBoneWeight.y+
					 BoneTransform(aBoneId.z,pos)*aBoneWeight.z+
					 BoneTransform(aBoneId.w,pos)*aBoneWeight.w);

		gl_Position  = MatWorld * nPos; 
		mediump vec4  nNorm  =	
					(BoneTransform(aBoneId.x,vec4(aNormal,0))*aBoneWeight.x+
					 BoneTransform(aBoneId.y,vec4(aNormal,0))*aBoneWeight.y+
					 BoneTransform(aBoneId.z,vec4(aNormal,0))*aBoneWeight.z+
					 BoneTransform(aBoneId.w,vec4(aNormal,0))*aBoneWeight.w);
		vNormal = normalize(world3*nNorm.xyz); 
	#else 
		gl_Position  = MatWorld*pos;
		vNormal = normalize(world3*aNormal); 
	#endif 
	
	vPosition = gl_Position; 
	
	gl_Position =  MatViewProjection* gl_Position ;
	
	#ifdef _DISTANCE_FOG
		vDistFog=ComputeDistanceFog(gl_Position.z,FogParams) ;
	#endif
}

#endif 

#ifdef FS 

uniform  mediump sampler2D TexDiffuse; 
uniform  mediump sampler2D TexSelf; 
uniform  mediump sampler2D TexNormal; 
uniform  mediump samplerCube TexEnv;
uniform  sampler2D TexLightMap;


void main(void) 
{
	lowp vec4 color =  matColor*texture2D(TexDiffuse, vTexCoord0);
	mediump vec3 N 		 =  normalize(vNormal);
	lowp vec3 EyeVec 	 =  normalize(camPos-vPosition.xyz);
	
		
	#ifdef _TexSelf
		lowp vec4 selfColor=texture2D(TexSelf, vTexCoord0)*SELF_MULTIPLY;
	#endif
	
	#ifdef _TexEnv
		lowp vec4 envColor =textureCube(TexEnv,-reflect(EyeVec,N)) *matParams2.x;
	#endif 
		
	#ifdef _ALPHA_BLEND
		color.rgb*=color.a; //premultipled alpha
	#else  //TODO: DECAL 
		lowp vec3 d,s;
		
		ComputeLighting(N,EyeVec,d,s);
		color.rgb *=  ambientColor.rgb + d; 
		color.rgb +=  s;
	#endif
	
	#ifdef _TexSelf
		color.rgb+=selfColor.rgb;
	#endif
	
	#ifdef _TexEnv
		color.rgb+=envColor.rgb;
	#endif
	
	#ifdef _DISTANCE_FOG
		lowp vec3 fogColor= FogColor;
		#ifdef _ALPHA_BLEND
			fogColor.rgb *= color.a; // premultiplied alpha! (lub additive)
		#endif
		color.rgb = mix(color.rgb, fogColor, vDistFog);
	#endif
	
	gl_FragColor = color;
	#ifdef _TexLightMap 
		#ifdef _TEX1
			gl_FragColor.rgb*=texture2D(TexLightMap, vTexCoord1).x;
		#endif
	#endif
	
}


#endif
