#pragma include "common_vs.glh"
#pragma include "common_fs.glh"

varying mediump vec3 vNormal; 
varying mediump vec2 vTexCoord0; 

#ifdef _TEX1
varying mediump vec2 vTexCoord1; 
#endif
varying mediump vec4 vPosition;
#ifdef _COLOR 
varying lowp vec4 vColor;
#endif
#ifdef _DISTANCE_FOG
varying lowp float vDistFog;
#endif

varying mediump vec3 vWorldTangentS;

#ifdef _TGNORMAL
	varying mediump vec3 vWorldTangentT;
#endif 

#ifdef VS

uniform  highp mat4 MatWorld; // 4 -> 124
uniform  highp mat4 MatViewProjection; // 4 -> 120
#ifdef _DUDV 
uniform  highp vec4 DuDvSpeed; 
#endif
uniform  highp vec4 Time; //TODO floata dac

void main(void) 
{
	mediump vec4 pos = aPosition;
	pos.w=1.0;
	
	vTexCoord0 = aTexCoord0;
	#ifdef _COLOR
		vColor=aColor;
	#endif

	#ifdef _TEX1
		vTexCoord1 =aTexCoord1;
	#endif
	
	#ifdef _DUDV 
		vTexCoord0+=mod(0.1*Time.xx,2.0);
	#endif 
	
	#ifdef _TGNORMAL
		    vWorldTangentS=aTangent;
			if(aTangent.x > 0.0) {
				vWorldTangentS.x = (aTangent.x - 0.5) * 2.0;
			}
			else{
				vWorldTangentS.x = (aTangent.x + 0.5) * 2.0;
			}
	#endif
	
	mediump mat3 world3 = mat3( MatWorld[0].xyz,MatWorld[1].xyz,MatWorld[2].xyz );
	
	#ifdef _ANIM
		mediump vec4  nPos =  
					(BoneTransform(aBoneId.x,pos)*aBoneWeight.x+
					 BoneTransform(aBoneId.y,pos)*aBoneWeight.y+
					 BoneTransform(aBoneId.z,pos)*aBoneWeight.z+
					 BoneTransform(aBoneId.w,pos)*aBoneWeight.w);

		gl_Position  = MatWorld * nPos; 
		mediump vec4  nNorm  =	
					(BoneTransform(aBoneId.x,vec4(aNormal,0))*aBoneWeight.x+
					 BoneTransform(aBoneId.y,vec4(aNormal,0))*aBoneWeight.y+
					 BoneTransform(aBoneId.z,vec4(aNormal,0))*aBoneWeight.z+
					 BoneTransform(aBoneId.w,vec4(aNormal,0))*aBoneWeight.w);
		vNormal = normalize(world3*nNorm.xyz); 
		

		#ifdef _TGNORMAL
			 
				mediump vec4  nTang  =	
						(BoneTransform(aBoneId.x,vec4(vWorldTangentS,0))*aBoneWeight.x+
						 BoneTransform(aBoneId.y,vec4(vWorldTangentS,0))*aBoneWeight.y+
						 BoneTransform(aBoneId.z,vec4(vWorldTangentS,0))*aBoneWeight.z+
						 BoneTransform(aBoneId.w,vec4(vWorldTangentS,0))*aBoneWeight.w);
				vWorldTangentS = normalize(nTang.xyz*world3);
				vWorldTangentT = normalize(cross(vNormal, vWorldTangentS));
				if(aTangent.x <0.0 ) {vWorldTangentT*=-1.0;}
			 
		#endif
		
	#else 
		gl_Position  = MatWorld*pos;
		vNormal = normalize(world3*aNormal);
		#ifdef _TGNORMAL
				vWorldTangentS = (world3*vWorldTangentS.xyz);
				vWorldTangentT = normalize(cross(vWorldTangentS,vNormal));//sign(aTangent.x);
				if(aTangent.x > 0.0 ) {vWorldTangentT*=-1.0;}
		#endif
	#endif 
	
	vPosition = gl_Position; 
	gl_Position =  MatViewProjection* gl_Position;
	
	#ifdef _DISTANCE_FOG
		vDistFog=ComputeDistanceFog(gl_Position.z,FogParams) ;
	#endif
}

#endif 

#ifdef FS 

uniform mediump vec4 edgeGlowParams;//wykorzystywane tylko xy strength,scale

uniform mediump sampler2D TexDiffuse; 
uniform mediump sampler2D TexSelf; 
uniform mediump sampler2D TexNormal;
uniform samplerCube TexEnv;

//uniform sampler2D TexSpecularColor; 
//uniform sampler2D TexSpecularPower; 

#define EDGE_GLOW_STRENGTH edgeGlowParams.x  
#define EDGE_GLOW_SCALE edgeGlowParams.y

#ifdef _ALPHA_TEST
	uniform mediump float AlphaTestRef;
#endif


//const mediump vec3 Perception = vec3(0.2126, 0.7152, 0.0722);
void main(void) 
{

	#ifdef _TexNormal
		#ifdef _TANGENT  
			mediump vec3 nn = texture2D(TexNormal, vTexCoord0).rgb*2.0 -1.0;
				nn.xy *= BUMP_LEVEL;
				nn.y *= -1.0;
				nn = normalize(nn);

			mediump mat3 worldToTangentSpace = mat3(
				normalize(vWorldTangentS),
				normalize(vWorldTangentT),
				normalize(vNormal));
			mediump vec3 N =normalize(worldToTangentSpace*nn);


		#else
			mediump vec3 N = normalize(vNormal);
		#endif
	#else
			mediump vec3 N = normalize(vNormal);
	#endif 
	
	#ifdef _TexDiffuse 
		mediump vec4 color = texture2D(TexDiffuse, vTexCoord0);
	#else 
		mediump vec4 color = vec4(1.0,1.0,1.0,1.0);
	#endif 

	mediump vec3 EyeVec =  normalize(camPos-vPosition.xyz);

	#ifdef _COLOR
		color*=matColor*vColor;
	#else 
		color*=matColor;
	#endif

	#ifdef _TexSelf
		mediump vec4 selfColor=texture2D(TexSelf, vTexCoord0)*SELF_MULTIPLY *color.a;
	#endif
	
	#ifdef _TexEnv
		mediump vec4 envColor = textureCube(TexEnv, -reflect(EyeVec,N)) * color.a *matParams2.x ;
	#endif 

	

	
	#ifdef _ALPHA_BLEND
		color.rgb*=color.a; //premultipled alpha
	#else
		#ifdef _ALPHA_TEST
			AlphaTest(color.a,AlphaTestRef); 
		#endif

		mediump float edge_mul =0.0;
		#ifdef _EDGE_GLOW
			highp float vvv = dot(N, EyeVec);
			edge_mul =  EDGE_GLOW_STRENGTH* pow( clamp(1.0-abs(vvv)/EDGE_GLOW_SCALE,0.0,1.0),2.0);
		#endif

		mediump vec3 d,s;
		//mediump vec3 bd,bs;
		ComputeLighting(N,EyeVec,d,s);
		//ComputeBackLighting(N,EyeVec,bd,bs);

		color.rgb *= ambientColor.rgb+ matParams2.y +d+d*edge_mul;
		color.rgb += s;	
	#endif
	

	
	
	#ifdef _TexSelf
		color.rgb+=selfColor.rgb;
	#endif
	
	#ifdef _TexEnv
		color.rgb +=envColor.rgb;
	#endif

	#ifdef _DISTANCE_FOG
		mediump vec4 fogColor= FogColor;
		#ifdef _ALPHA_BLEND
			fogColor.rgb *= color.a; // premultiplied alpha! (lub additive)
		#endif
		color.rgb = mix(color.rgb, fogColor.rgb, vDistFog);		
	#endif

	// Do debugowania
	#ifdef __SHOW_N
		#ifndef _ALPHA_BLEND
			color.rgb = N*0.5+0.5;
			color.a =1.0;
		#else
			color.a =0.0;
		#endif
	#endif
	#ifdef __SHOW_LIGHT
		#ifndef _ALPHA_BLEND
			color.rgb = d;
			//color.rgb += s;
			color.a =1.0;
		#else
			color.a =0.0;
		#endif
	#endif
 
 	
  	gl_FragColor=color;
}


#endif