
varying mediump vec3 vNormal; 
varying mediump vec2 vTexCoord0; 
varying mediump vec4 vPosition; 
varying lowp vec4 vColor;

#ifdef VS

attribute vec4 aPosition; 
attribute vec3 aNormal;
attribute vec2 aTexCoord0; 
attribute vec4 aColor;

#ifdef _ANIM
	attribute vec4 aBoneWeight; 
	attribute vec4 aBoneId; 
#endif

uniform  mediump mat4 MatWorld; // 4 -> 124
uniform  mediump mat4 MatViewProjection; // 4 -> 120
uniform  mediump vec4 DuDvSpeed; 
uniform  mediump vec4 Time; //TODO floata dac

#ifdef _ANIM
	uniform  mediump vec4 Bones[32*3]; // 24 vecs left
	
	vec4 BoneTransform(in float id, in vec4 pos) 
	{
		return  vec4(dot(pos,Bones[int(id)*3]),dot(pos,Bones[int(id)*3+1]),dot(pos,Bones[int(id)*3+2]),1.0);
	} 
	
#endif


void main(void) 
{
	mediump vec4 pos = aPosition;
	pos.w=1.0;
	
	vTexCoord0 = aTexCoord0;
	#ifdef _DUDV 
		vTexCoord0+=mod(DuDvSpeed.xy*Time.xx,2.0);
	#endif 
	
	mediump mat3 world3 = mat3( MatWorld[0].xyz,MatWorld[1].xyz,MatWorld[2].xyz );
	
	#ifdef _ANIM
		mediump vec4  nPos =  
					(BoneTransform(aBoneId.x,pos)*aBoneWeight.x+
					 BoneTransform(aBoneId.y,pos)*aBoneWeight.y+
					 BoneTransform(aBoneId.z,pos)*aBoneWeight.z+
					 BoneTransform(aBoneId.w,pos)*aBoneWeight.w);

		gl_Position  = MatWorld * nPos; 
		mediump vec4  nNorm  =	
					(BoneTransform(aBoneId.x,vec4(aNormal,0))*aBoneWeight.x+
					 BoneTransform(aBoneId.y,vec4(aNormal,0))*aBoneWeight.y+
					 BoneTransform(aBoneId.z,vec4(aNormal,0))*aBoneWeight.z+
					 BoneTransform(aBoneId.w,vec4(aNormal,0))*aBoneWeight.w);
		vNormal = normalize(world3*nNorm.xyz); 
	#else 
		gl_Position  = MatWorld*pos;
		vNormal = normalize(world3*aNormal); 
	#endif 
	
	vPosition = gl_Position; 
	vColor=aColor;
	gl_Position =  MatViewProjection* gl_Position ;
}

#endif 
#ifdef FS 


uniform lowp vec3 sunDir[3];
uniform lowp vec4 sunColor[3];
uniform int lights; 
uniform lowp vec4 ambientColor;
uniform lowp vec4 matColor; 

uniform mediump vec4 matParams;
uniform mediump vec4 matParams2;
uniform mediump vec4 edgeGlowParams;//wykorzystywane tylko xy strength,scale
uniform mediump vec3 camPos; 

uniform sampler2D TexDiffuse; 
uniform sampler2D TexSelf; 
uniform sampler2D TexSpecularColor; 
uniform sampler2D TexSpecularPower; 
uniform  lowp samplerCube TexEnv;
//const mediump vec3 Perception = vec3(0.299, 0.587, 0.114);



#define EDGE_GLOW_STRENGTH edgeGlowParams.x  
#define EDGE_GLOW_SCALE edgeGlowParams.y

//INCLUDE_COMMON_FS 
#define SELF_MULTIPLY matParams.x
#define SPECULAR_LEVEL matParams.y
#define BUMP_LEVEL matParams.z
#define GLOSSINESS matParams.w 

void  ComputeLighting(in mediump  vec3 N,in mediump vec3 EyeVec, out lowp vec3 diffLight, out lowp  vec3 specLight) 
{ 

	diffLight= vec3(0.0);
	specLight= vec3(0.0);
	#ifdef _LIGHT 
	//for(int i=0 ; i<lights;i++ )
	int i=0;
	{
		lowp float NdotL = clamp(dot(N,-sunDir[i]),0.0,1.0);
		mediump float spec= 0.0;
		if (NdotL > 0.0 )
		{
			mediump float NdotH = dot(N, normalize(EyeVec-sunDir[i]));
			//narazie bez
			if(NdotH>0.0) 
			{	
				spec=pow(NdotH,exp2(GLOSSINESS * 10.0)); // *10 na cpu i to 0.75* + 0.25
			}	
		}
		diffLight += NdotL*sunColor[i].rgb;
		specLight += SPECULAR_LEVEL*sunColor[i].rgb*spec;
	}	
	#endif
} 
//INCLUDE_COMMON_FS 

void main(void) 
{
	lowp vec4 color 	 =  texture2D(TexDiffuse, vTexCoord0);
	lowp vec3 EyeVec 	 =  normalize(camPos-vPosition.xyz);
	mediump vec3 N 		 =  normalize(vNormal);
	
	#ifdef _TexSelf
		lowp vec4 selfColor = vec4(0.0);
		selfColor=texture2D(TexSelf, vTexCoord0)*SELF_MULTIPLY;//self multiply
	#endif
	
	#ifdef _TexEnv
		lowp vec4 envColor =textureCube(TexEnv,-reflect(EyeVec,N)) *matParams2.x;
	#endif 
	
	lowp float edge_mul =0.0;
	#ifdef _EDGE_GLOW
		lowp float vvv = dot(N, EyeVec);
		edge_mul = EDGE_GLOW_STRENGTH * pow( clamp(1.0-abs(vvv)/EDGE_GLOW_SCALE,0.0,1.0),2.0);
	#endif
	
	
	color *= matColor;
	
	#ifdef _ALPHA_BLEND
		color.rgb*=color.w; //premultipled alpha
	#else  //TODO: DECAL 
			lowp vec3 d,s;
			ComputeLighting(N,EyeVec,d,s);
			color.rgb *=  ambientColor.rgb + d + edge_mul; 
			color.rgb +=  s;
	#endif
		
	#ifdef _TexSelf
		color.rgb+=selfColor.rgb;
	#endif
	#ifdef _TexEnv
		color.rgb+=envColor.rgb;
	#endif
	
	
	gl_FragColor=color; 
}


#endif