
varying lowp vec3 vNormal; 
varying lowp vec2 vTexCoord0; 
varying mediump vec4 vPosition; 
varying mediump float vDist;
varying lowp vec4 vColor;
varying lowp vec3 vEyeVec;

#ifdef VS

attribute vec4 aPosition; 
attribute vec3 aNormal;
attribute vec2 aTexCoord0; 
attribute vec4 aColor;

uniform  mediump mat4 MatWorld; // 4 -> 124
uniform  mediump mat4 MatViewProjection; // 4 -> 120

uniform  mediump vec3 camPos;
uniform  mediump vec4 DuDvSpeed; 
uniform  mediump vec4 Time; //TODO floata dac

void main(void) 
{
	mediump vec4 pos = aPosition;
	pos.w=1.0;
	vTexCoord0 = aTexCoord0;
	
	#ifdef _DUDV 
		vTexCoord0+=mod(DuDvSpeed.xy*Time.xx,2.0);
	#endif 
	
	mediump mat3 world3 = mat3( MatWorld[0].xyz,MatWorld[1].xyz,MatWorld[2].xyz );
	
	vPosition = MatWorld*pos; 
	vColor=aColor;
	vNormal = normalize(world3*aNormal); 
	vEyeVec  = normalize(camPos-vPosition.xyz);
	gl_Position =  MatViewProjection* vPosition;
	vDist = gl_Position.z;
}

#endif 
#ifdef FS 

uniform lowp vec3 sunDir[3];
uniform lowp vec4 sunColor[3];
uniform int lights; 

uniform lowp vec4 ambientColor;
uniform lowp vec4 matColor; 

uniform lowp vec4 matParams;
uniform mediump vec4 matParams2;
uniform mediump vec4 FadeParams;//xy dist0 dist1 
uniform mediump vec3 camPos; 

uniform lowp sampler2D TexDiffuse; 

#ifdef _TexMask
	uniform sampler2D TexMask; 
#endif

#define DIST0  FadeParams.x 
#define DIST1  FadeParams.y



//INCLUDE_COMMON_FS - TODO przenie do wsplnego pliku
#define SELF_MULTIPLY matParams.x
#define SPECULAR_LEVEL matParams.y
#define BUMP_LEVEL matParams.z
#define GLOSSINESS matParams.w 

void  ComputeLighting(in mediump  vec3 N,in mediump vec3 EyeVec, out lowp vec3 diffLight, out lowp  vec3 specLight) 
{ 
	diffLight= vec3(0.0);
	specLight= vec3(0.0);
	#ifdef _LIGHT 
	//for(int i=0 ; i<lights;i++ )
	int i=0;
	{
		lowp float NdotL = clamp(dot(N,-sunDir[i]),0.0,1.0);
		mediump float spec= 0.0;
		if (NdotL > 0.0 )
		{
			mediump float NdotH = dot(N, normalize(EyeVec-sunDir[i]));
			//narazie bez
			/*if(NdotH>0.0) 
			{	
				spec=pow(NdotH,exp2(GLOSSINESS * 10.0)); // *10 na cpu i to 0.75* + 0.25
			}*/	
		}
		diffLight += NdotL*sunColor[i].rgb;
		specLight += SPECULAR_LEVEL*sunColor[i].rgb*spec;
	}	
	#endif
} 
//INCLUDE_COMMON_FS 


void main(void) 
{
	lowp vec4 color =  matColor*texture2D(TexDiffuse, vTexCoord0);
	mediump vec3 N 		 =  normalize(vNormal);
	
	#ifdef _FADE_PARAMS
		lowp float ndot = abs(dot(N, vEyeVec));
		color.a *= ndot*ndot;
		color.a *= clamp((vDist - DIST0) / (DIST1 - DIST0),0.0,1.0);
	#endif
		
	#ifdef _TexMask
		color.a *= texture2D(TexMask, vTexCoord0).a;
	#endif 
	
	
	#ifdef _ALPHA_BLEND
		color.rgb*=color.w; //premultipled alpha
	#else  //TODO: DECAL 
		lowp vec3 d,s;
		ComputeLighting(N,vEyeVec,d,s);
		color.rgb *=  ambientColor.rgb + d; 
		color.rgb +=  s;
	#endif
	
	
	gl_FragColor=color; 
}


#endif