#pragma include "common_vs.glh"

varying lowp vec2 vTexCoord0; 
#ifdef _TEX1
varying lowp vec2 vTexCoord1; 
#endif

#ifdef VS

uniform  mediump mat4 MatWorld; // 4 -> 124
uniform  mediump mat4 MatViewProjection; // 4 -> 120
//uniform  mediump vec4 DuDvSpeed; 
uniform  mediump vec4 Time; //TODO floata dac
uniform  mediump vec4 VertexDeformParams;


#define vd_Speed VertexDeformParams.x
#define vd_DetailFreq VertexDeformParams.y
#define vd_DetailAmp VertexDeformParams.z
#define vd_BranchAmp VertexDeformParams.w


vec4 SmoothCurve( vec4 x ) {  
   return x * x *( 3.0 - 2.0 * x );  
}  

vec4 TriangleWave( vec4 x ) {  
   return abs( fract( x + 0.5 ) * 2.0 - 1.0 );  
}  

vec4 SmoothTriangleWave( vec4 x ) {  
   return SmoothCurve( TriangleWave( x ) );  
}

void main(void) 
{
	mediump vec4 pos = aPosition;
	pos.w=1.0;
	lowp vec3 vNormal; 
	#ifdef _COLOR
		lowp vec4 vColor=aColor;
	#endif
	
	vTexCoord0 = aTexCoord0;
	
	#ifdef _TEX1
		vTexCoord1 =aTexCoord1;
	#endif
	

	mediump mat3 world3 = mat3( MatWorld[0].xyz,MatWorld[1].xyz,MatWorld[2].xyz );
	
	#ifdef _ANIM
		mediump vec4  nPos =  
					(BoneTransform(aBoneId.x,pos)*aBoneWeight.x+
					 BoneTransform(aBoneId.y,pos)*aBoneWeight.y+
					 BoneTransform(aBoneId.z,pos)*aBoneWeight.z+
					 BoneTransform(aBoneId.w,pos)*aBoneWeight.w);

		gl_Position  = nPos;// MatWorld * nPos; 
		mediump vec4  nNorm  =	
					(BoneTransform(aBoneId.x,vec4(aNormal,0))*aBoneWeight.x+
					 BoneTransform(aBoneId.y,vec4(aNormal,0))*aBoneWeight.y+
					 BoneTransform(aBoneId.z,vec4(aNormal,0))*aBoneWeight.z+
					 BoneTransform(aBoneId.w,vec4(aNormal,0))*aBoneWeight.w);
		vNormal = normalize(world3*nNorm.xyz); 
	#else 
		gl_Position  =pos; //MatWorld*pos;
		vNormal = normalize(world3*aNormal); 
	#endif 
	
	// to sie nie powinno stac ... (brak _COLOR)
	#ifdef _COLOR		
	float fBranchPhase = vColor.g;
	float fDetailPhase = 0.0; // ...
	float fSpeed = vd_Speed;
	float fDetailFreq = vd_DetailFreq;
	float fEdgeAtten = 1.0-vColor.r;
	float fBranchAtten = 1.0-vColor.b;
	float fDetailAmp = vd_DetailAmp;
	float fBranchAmp = vd_BranchAmp;
	
	// Phases (object, vertex, branch)  
    float fObjPhase = dot(vec3(MatWorld[0].w, MatWorld[1].w, MatWorld[2].w), vec3(1.0));  
 	fBranchPhase += fObjPhase;  
	float fVtxPhase = dot(aPosition.xyz, vec3(fDetailPhase + fBranchPhase));  
 	// x is used for edges; y is used for branches  
    vec2 vWavesIn = Time.xx + vec2(fVtxPhase, fBranchPhase );  
 	// 1.975, 0.793, 0.375, 0.193 are good frequencies  
    vec4 vWaves = (  fract( vWavesIn.xxyy *  
                        vec4(1.975, 0.793, 0.375, 0.193) ) *  
                        2.0 - 1.0 ) * fSpeed * fDetailFreq;  
 	vWaves = SmoothTriangleWave( vWaves );  
 	vec2 vWavesSum = vWaves.xz + vWaves.yw;  
 	// Edge (xy) and branch bending (z)  
 	gl_Position.xyz += vWavesSum.xxy * vec3(fEdgeAtten * fDetailAmp * vNormal.xy, fBranchAtten * fBranchAmp);  
	#endif
	gl_Position =  MatViewProjection*MatWorld* gl_Position;
}

#endif 
#ifdef FS 

//uniform lowp vec4 ambientColor;
uniform lowp vec4 matColor; 

//uniform mediump vec4 matParams;
//uniform mediump vec4 matParams2;
//uniform  mediump vec3 camPos; 

//uniform  lowp samplerCube TexEnv;
uniform  lowp sampler2D TexDiffuse; 
//uniform  sampler2D TexSelf; 

const mediump vec3 Perception = vec3(0.299, 0.587, 0.114);
void main(void) 
{
	lowp vec4 color =  matColor*texture2D(TexDiffuse, vTexCoord0);
	gl_FragColor= color;
	if(gl_FragColor.a < 0.5) { discard;}
}


#endif