function nop()
end

function rprint(t, indent, key)
	indent = indent or ""
	if type(t) == "table" then
		print(indent .. (key or "") .. 'table(')
		local subindent = indent .. "  "
		for k,v in pairs(t) do
			rprint(v, subindent, '[' .. tostring(k) .. '] = ')
		end

		local meta = getmetatable(t)
		if meta then
			rprint(meta, subindent, 'metatable = ')
		end
		print(indent .. ')')
	else
		print(indent .. (key or "") .. tostring(t))
	end
end

require("lua.math")
scheduler = require("lua.scheduler")
hook_table = require('lua.hooks')

function missing(self, name, ...)
	print('missing method ' .. name .. ' was called on ' .. (self.rootnode and self.rootnode.name or self))
	self[name] = nop
	return nop
end

function send_message(obj, name, ...)
	local script = obj and obj.script
	if script then
		local handler = script[name] or missing(script, name)
		handler(script, ...)
	else
		error('message send failed ' .. name)-- obj, name)
	end
end

local _G = _G

function define_global(name, value)
	rawset(_G, name, value)
end

local static = {}
local main_sched = scheduler()

function static:step_scheduler(dt)
	main_sched.work(dt)
end

run_later = main_sched.run_later

function set_timer(dt, callable)
	local iterator
	local delta = dt
	local function looprunner()
		if callable() then return end
		iterator = run_later(delta, looprunner)
	end
	iterator = run_later(delta, looprunner)

	local rv
	rv = {
		stop = function()
			main_sched.unschedule(iterator)
			rv.stop = nil
			rv.set_delta = nil
		end,
		set_delta = function(dt)
			delta = dt
		end
	}
	return rv
end

function start_microthread(arg1, arg2)
	local fun
	if type(arg1) == "function" then
		fun = arg1
	else
		local obj = arg1
		local method = arg2 or "thread"
		fun = function() obj[method](obj) end
	end
	local co = coroutine.create(fun)
	local rv, err = coroutine.resume(co)
	if not rv then
		error('coroutine error:'..err)
	end
end

function sleep(dt)
	local co = coroutine.running()
	if not co then error("can't call sleep from main thread") end
	run_later(dt, function() 
		local rv,err = coroutine.resume(co) 
		if not rv then
			error('coroutine error:'..err)
		end
	end)
	coroutine.yield()
end


-- end of globals

setmetatable(_G, { 
		__index = function(t,k) 
			error("undefined global \"" .. k .. "\"\n" .. debug.traceback()) 
		end,
		__newindex = function(t,k,v) 
			error("please don't set globals - error when setting \"" .. k .. "\"\n" .. debug.traceback()) 
		end
	})

return static
