function get_translation(m)
	return { m[4], m[8], m[12] }
end

function get_scale(m)
	return vec_length({m[1], m[5], m[9] })
end

function vec_dot(a,b)
	return a[1]*b[1] + a[2]*b[2] + a[3]*b[3]
end

function vec_cross(a,b)
	return {
		a[2]*b[3] - a[3]*b[2],
		a[3]*b[1] - a[1]*b[3],
		a[1]*b[2] - a[2]*b[1]
	}
end

function vec_add(a,b)
	return {a[1]+b[1], a[2]+b[2], a[3]+b[3]}
end

function vec_sub(a,b)
	return {a[1]-b[1], a[2]-b[2], a[3]-b[3]}
end

function vec_muls(a, s)
	return {a[1]*s, a[2]*s, a[3]*s}
end

function vec_length(v)
	return math.sqrt(vec_dot(v,v))
end

function vec_normalize(v)
	local l = vec_length(v)
	return { v[1]/l, v[2]/l, v[3]/l }
end

function mtx_translate(offset)
	return { 1, 0, 0, offset[1],
		0, 1, 0, offset[2],
		0, 0, 1, offset[3] }
end

function mtx_pt_normal(pt, normal)
	local b1, b2
	if math.abs(normal[1]) > 0.5 then
		b1 = vec_cross(normal, { 0, 1, 0 })
	else
		b1 = vec_cross(normal, { 1, 0, 0 })
	end
	b1 = vec_normalize(b1)
	b2 = vec_normalize(vec_cross(b1, normal))
	return {b1[1], b2[1], normal[1], pt[1],
		b1[2], b2[2], normal[2], pt[2],
		b1[3], b2[3], normal[3], pt[3] }
end

mtx_inverse = live.mtx_inverse
mtx_lookat = live.mtx_lookat

function mtx_mul(a, b)
	return {
		a[1] * b[1] + a[2] * b[5] + a[3] * b[9],
		a[1] * b[2] + a[2] * b[6] + a[3] * b[10],
		a[1] * b[3] + a[2] * b[7] + a[3] * b[11],
		a[1] * b[4] + a[2] * b[8] + a[3] * b[12] + a[4],

		a[5] * b[1] + a[6] * b[5] + a[7] * b[9],
		a[5] * b[2] + a[6] * b[6] + a[7] * b[10],
		a[5] * b[3] + a[6] * b[7] + a[7] * b[11],
		a[5] * b[4] + a[6] * b[8] + a[7] * b[12] + a[8],

		a[9] * b[1] + a[10] * b[5] + a[11] * b[9],
		a[9] * b[2] + a[10] * b[6] + a[11] * b[10],
		a[9] * b[3] + a[10] * b[7] + a[11] * b[11],
		a[9] * b[4] + a[10] * b[8] + a[11] * b[12] + a[12] }
end

function mtx_mulv0(a, b)
	return {
			a[1] * b[1] + a[2] * b[2] + a[3] * b[3],
			a[5] * b[1] + a[6] * b[2] + a[7] * b[3],
			a[9] * b[1] + a[10]* b[2] + a[11]* b[3]
	}
end

function mtx_mulv1(a, b)
	return {
			a[1] * b[1] + a[2] * b[2] + a[3] * b[3] + a[4],
			a[5] * b[1] + a[6] * b[2] + a[7] * b[3] + a[8],
			a[9] * b[1] + a[10]* b[2] + a[11]* b[3] + a[12]
	}
end

function vec_lerp(a,b,v)
	local omv = 1-v
	return {
		a[1] * omv + b[1] * v,
		a[2] * omv + b[2] * v,
		a[3] * omv + b[3] * v }
end

function mtx_lerp(a,b,v)
	local omv = 1-v
	return {
		a[1] * omv + b[1] * v,
		a[2] * omv + b[2] * v,
		a[3] * omv + b[3] * v,
		a[4] * omv + b[4] * v,
		a[5] * omv + b[5] * v,
		a[6] * omv + b[6] * v,
		a[7] * omv + b[7] * v,
		a[8] * omv + b[8] * v,
		a[9] * omv + b[9] * v,
		a[10] * omv + b[10] * v,
		a[11] * omv + b[11] * v,
		a[12] * omv + b[12] * v }
end

mtx_mulv = mtx_mulv1


