local function new_scheduler()
	local self = {}
	local head = {}
	local timer = 0

	local function insert(pred, time, callable)
		local entry = {
			time = time,
			callable = callable
		}

		entry.next = pred.next
		pred.next = entry
		return entry
	end

	function self.run_later(dt, callable)
		local time = timer + dt

		local pred = head
		local tmp = head.next
		while tmp and tmp.time < time do
			pred = tmp
			tmp = tmp.next
		end

		return insert(pred, time, callable)
	end

	function self.unschedule(cur)
		local pred = head
		local tmp = head.next
		while tmp do
			if tmp == cur then
				cur.callable = function() error("stale task") end
				pred.next = cur.next
				return
			end

			pred = tmp
			tmp = tmp.next
		end
	end

	function self.work(dt)
		timer = timer + dt
		while head.next do
			local it = head.next
			if it.time < timer then
				head.next = it.next
				it.callable()
			else
				break
			end
		end
	end

	return self
end

return new_scheduler
